@@ -32,8 +32,23 @@
 
 static DBusConnection *g_connection;
 
+struct error_mapping_entry {
+	int error;
+	DBusMessage *(*ofono_error_func)(DBusMessage *);
+};
+
+struct error_mapping_entry cme_errors_mapping[] = {
+	{ 3,	__ofono_error_not_allowed },
+	{ 4,	__ofono_error_not_supported },
+	{ 16,	__ofono_error_incorrect_password },
+	{ 30,	__ofono_error_not_registered },
+	{ 31,	__ofono_error_timed_out },
+	{ 32,	__ofono_error_access_denied },
+	{ 50,	__ofono_error_invalid_args },
+};
+
 static void append_variant(DBusMessageIter *iter,
-				int type, void *value)
+				int type, const void *value)
 {
 	char sig[2];
 	DBusMessageIter valueiter;
@@ -50,7 +65,7 @@ static void append_variant(DBusMessageIter *iter,
 }
 
 void ofono_dbus_dict_append(DBusMessageIter *dict,
-			const char *key, int type, void *value)
+			const char *key, int type, const void *value)
 {
 	DBusMessageIter keyiter;
 
@@ -70,7 +85,8 @@ void ofono_dbus_dict_append(DBusMessageIter *dict,
 	dbus_message_iter_close_container(dict, &keyiter);
 }
 
-static void append_array_variant(DBusMessageIter *iter, int type, void *val)
+static void append_array_variant(DBusMessageIter *iter, int type,
+				const void *val)
 {
 	DBusMessageIter variant, array;
 	char typesig[2];
@@ -98,7 +114,7 @@ static void append_array_variant(DBusMessageIter *iter, int type, void *val)
 }
 
 void ofono_dbus_dict_append_array(DBusMessageIter *dict, const char *key,
-				int type, void *val)
+				int type, const void *val)
 {
 	DBusMessageIter entry;
 
@@ -112,7 +128,8 @@ void ofono_dbus_dict_append_array(DBusMessageIter *dict, const char *key,
 	dbus_message_iter_close_container(dict, &entry);
 }
 
-static void append_dict_variant(DBusMessageIter *iter, int type, void *val)
+static void append_dict_variant(DBusMessageIter *iter, int type,
+				const void *val)
 {
 	DBusMessageIter variant, array, entry;
 	char typesig[5];
@@ -167,7 +184,7 @@ static void append_dict_variant(DBusMessageIter *iter, int type, void *val)
 }
 
 void ofono_dbus_dict_append_dict(DBusMessageIter *dict, const char *key,
-				int type, void *val)
+				int type, const void *val)
 {
 	DBusMessageIter entry;
 
@@ -185,7 +202,7 @@ int ofono_dbus_signal_property_changed(DBusConnection *conn,
 					const char *path,
 					const char *interface,
 					const char *name,
-					int type, void *value)
+					int type, const void *value)
 {
 	DBusMessage *signal;
 	DBusMessageIter iter;
@@ -210,7 +227,7 @@ int ofono_dbus_signal_array_property_changed(DBusConnection *conn,
 						const char *path,
 						const char *interface,
 						const char *name,
-						int type, void *value)
+						int type, const void *value)
 
 {
 	DBusMessage *signal;
@@ -236,7 +253,7 @@ int ofono_dbus_signal_dict_property_changed(DBusConnection *conn,
 						const char *path,
 						const char *interface,
 						const char *name,
-						int type, void *value)
+						int type, const void *value)
 
 {
 	DBusMessage *signal;
@@ -349,6 +366,13 @@ DBusMessage *__ofono_error_attach_in_progress(DBusMessage *msg)
 				"GPRS Attach is in progress");
 }
 
+DBusMessage *__ofono_error_not_registered(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg,
+				OFONO_ERROR_INTERFACE ".NotRegistered",
+				"Modem is not registered to the network");
+}
+
 DBusMessage *__ofono_error_canceled(DBusMessage *msg)
 {
 	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".Canceled",
@@ -368,6 +392,60 @@ DBusMessage *__ofono_error_emergency_active(DBusMessage *msg)
 				"Emergency mode active");
 }
 
+DBusMessage *__ofono_error_incorrect_password(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg,
+				OFONO_ERROR_INTERFACE ".IncorrectPassword",
+				"Password is incorrect");
+}
+
+DBusMessage *__ofono_error_not_allowed(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".NotAllowed",
+					"Operation is not allowed");
+}
+
+DBusMessage *__ofono_error_not_recognized(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".NotRecognized",
+					"String not recognized as USSD/SS");
+}
+
+DBusMessage *__ofono_error_network_terminated(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE
+					".Terminated",
+					"Operation was terminated by the"
+					" network");
+}
+
+DBusMessage *__ofono_error_from_error(const struct ofono_error *error,
+						DBusMessage *msg)
+{
+	struct error_mapping_entry *e;
+	int maxentries;
+	int i;
+
+	switch (error->type) {
+	case OFONO_ERROR_TYPE_CME:
+		e = cme_errors_mapping;
+		maxentries = sizeof(cme_errors_mapping) /
+					sizeof(struct error_mapping_entry);
+		for (i = 0; i < maxentries; i++)
+			if (e[i].error == error->error)
+				return e[i].ofono_error_func(msg);
+		break;
+	case OFONO_ERROR_TYPE_CMS:
+		return __ofono_error_failed(msg);
+	case OFONO_ERROR_TYPE_CEER:
+		return __ofono_error_failed(msg);
+	default:
+		return __ofono_error_failed(msg);
+	}
+
+	return __ofono_error_failed(msg);
+}
+
 void __ofono_dbus_pending_reply(DBusMessage **msg, DBusMessage *reply)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
