@@ -1,21 +1,21 @@
 /*
- * oFono - GSM Telephony Stack for Linux
  *
- * Copyright (C) 2008-2010 Intel Corporation.  All rights reserved.
+ *  oFono - Open Source Telephony
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -155,7 +155,7 @@ static void vcard_printf_number(GString *vcards, const char *number, int type,
 	char *pref = "", *intl = "", *category_string = "";
 	char buf[128];
 
-	if (!number || !strlen(number) || !type)
+	if (number == NULL || !strlen(number) || !type)
 		return;
 
 	switch (category) {
@@ -233,23 +233,28 @@ static void vcard_printf_end(GString *vcards)
 	vcard_printf(vcards, "");
 }
 
-static void print_number(struct phonebook_number *pn, GString *vcards)
+static void print_number(gpointer pointer, gpointer user_data)
 {
+	struct phonebook_number *pn = pointer;
+	GString *vcards = user_data;
 	vcard_printf_number(vcards, pn->number, pn->type, pn->category);
 }
 
-static void destroy_number(struct phonebook_number *pn)
+static void destroy_number(gpointer pointer)
 {
+	struct phonebook_number *pn = pointer;
 	g_free(pn->number);
 	g_free(pn);
 }
 
-static void print_merged_entry(struct phonebook_person *person, GString *vcards)
+static void print_merged_entry(gpointer pointer, gpointer user_data)
 {
+	struct phonebook_person *person = pointer;
+	GString *vcards = user_data;
 	vcard_printf_begin(vcards);
 	vcard_printf_text(vcards, person->text);
 
-	g_slist_foreach(person->number_list, (GFunc) print_number, vcards);
+	g_slist_foreach(person->number_list, print_number, vcards);
 
 	vcard_printf_group(vcards, person->group);
 	vcard_printf_email(vcards, person->email);
@@ -257,15 +262,15 @@ static void print_merged_entry(struct phonebook_person *person, GString *vcards)
 	vcard_printf_end(vcards);
 }
 
-static void destroy_merged_entry(struct phonebook_person *person)
+static void destroy_merged_entry(gpointer pointer)
 {
+	struct phonebook_person *person = pointer;
 	g_free(person->text);
 	g_free(person->group);
 	g_free(person->email);
 	g_free(person->sip_uri);
 
-	g_slist_foreach(person->number_list, (GFunc) destroy_number, NULL);
-	g_slist_free(person->number_list);
+	g_slist_free_full(person->number_list, destroy_number);
 
 	g_free(person);
 }
@@ -277,8 +282,7 @@ static DBusMessage *generate_export_entries_reply(struct ofono_phonebook *pb,
 	DBusMessageIter iter;
 
 	reply = dbus_message_new_method_return(msg);
-
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -292,7 +296,7 @@ static gboolean need_merge(const char *text)
 	int len;
 	char c;
 
-	if (!text)
+	if (text == NULL)
 		return FALSE;
 
 	len = strlen(text);
@@ -375,7 +379,7 @@ void ofono_phonebook_entry(struct ofono_phonebook *phonebook, int index,
 				break;
 		}
 
-		if (!l) {
+		if (l == NULL) {
 			person = g_new0(struct phonebook_person, 1);
 			phonebook->merge_list =
 				g_slist_prepend(phonebook->merge_list, person);
@@ -420,10 +424,9 @@ static void export_phonebook_cb(const struct ofono_error *error, void *data)
 
 	/* convert the collected entries that are already merged to vcard */
 	phonebook->merge_list = g_slist_reverse(phonebook->merge_list);
-	g_slist_foreach(phonebook->merge_list, (GFunc) print_merged_entry,
+	g_slist_foreach(phonebook->merge_list, print_merged_entry,
 				phonebook->vcards);
-	g_slist_foreach(phonebook->merge_list, (GFunc) destroy_merged_entry,
-				NULL);
+	g_slist_free_full(phonebook->merge_list, destroy_merged_entry);
 	g_slist_free(phonebook->merge_list);
 	phonebook->merge_list = NULL;
 
@@ -444,8 +447,7 @@ static void export_phonebook(struct ofono_phonebook *phonebook)
 	}
 
 	reply = generate_export_entries_reply(phonebook, phonebook->pending);
-
-	if (!reply) {
+	if (reply == NULL) {
 		dbus_message_unref(phonebook->pending);
 		return;
 	}
@@ -481,13 +483,14 @@ static DBusMessage *import_entries(DBusConnection *conn, DBusMessage *msg,
 	return NULL;
 }
 
-static GDBusMethodTable phonebook_methods[] = {
-	{ "Import",	"",	"s",	import_entries,
-					G_DBUS_METHOD_FLAG_ASYNC },
+static const GDBusMethodTable phonebook_methods[] = {
+	{ GDBUS_ASYNC_METHOD("Import",
+			NULL, GDBUS_ARGS({ "entries", "s" }),
+			import_entries) },
 	{ }
 };
 
-static GDBusSignalTable phonebook_signals[] = {
+static const GDBusSignalTable phonebook_signals[] = {
 	{ }
 };
 
